// example.cpp : Defines the entry point for the console application.
//
//This example links to libmj2


#include <stdio.h>
#include <stdint.h>
#include <malloc.h>

#include "sci_types.h"
#include "sci_errors.h"
#include "sci_device.h"

#include "mj2_defs.h"
#include "mj2_regs.h"
#include "mj2_advance.h"
#include "mj2_functions.h"

// TODO: reference additional headers your program requires here


/* Some local functions to display results and perform memory allocation */
void display_device_status(device_status_t *device_status)
{
	printf("\n\n*********DEVICE STATUS READ FROM DEVICE***************\n");
	if (device_status->pll_status.vcxo_pll_ld)
		printf(" 100MHz VCXO reference is locked\n");
	else
		printf(" 100MHz reference is unlocked\n");

	if (device_status->pll_status.sum_pll_ld)
		printf(" RF main oscillator is locked \n");
	else
		printf(" RF main oscillator is unlocked \n");

	if (device_status->pll_status.crs_pll_ld)
		printf(" Coarse Pll Lock \n");
	else
		printf(" Coarse Pll Lock not locked \n");

	if (device_status->operate_status.pxi10_clk_enable)
		printf(" PXI 10 Mhz is exported \n");
	else
		printf(" PXI 10 MHZ is not exported \n");

	printf("The converter standby status = %d \n", device_status->operate_status.device_standby);
	printf("The external lock status = %d \n", device_status->operate_status.ext_lo_enable);
	printf("The external detect status = %d \n", device_status->operate_status.ext_ref_detected);
	printf("The synth LO mode = %d \n", device_status->operate_status.lo_mode);
	printf("The synth loop gain = %d \n", device_status->pll_status.loop_gain);
}

void display_device_info(device_info_t *dev_info)
{
	printf("\n**********DEVICE ATTRIBUTES************* \n");
	printf(" The product serial number is 0x%08X \n", dev_info->product_sn);
	printf(" The product firmware rev. is %f \n", dev_info->fw_rev);
	printf(" The product hardware rev. is %f \n", dev_info->hw_rev);
	printf(" The product manufacture date is: 20%d-%d-%d\n\n",
		dev_info->man_date.year, dev_info->man_date.month,
		dev_info->man_date.day);
}

void display_rf_parameters(rf_params_t *device_rf_params)
{
	printf("\n**********RF PARAMETERS************* \n");
	printf(" The current rf frequency is %f Hz\n", (double)device_rf_params->rf_frequency);
	printf(" The lo frequency is %f Hz\n", (double)device_rf_params->lo_frequency);
	printf(" The if frequency is %f Hz\n", (double)device_rf_params->if_frequency);
	printf(" The inverted spectrum state is %d \n", (int)device_rf_params->path_params.invert_spectrum);
}

void display_auto_gain_calc_params(gain_params_t* gain_params)
{
	printf("\n**********RF PARAMETERS************* \n");
	printf(" The RF nominal level is %f Hz\n", (float)gain_params->rf_level);
	printf(" The mixer level is %f Hz\n", (double)gain_params->mixer_level);
	printf(" The desired IF level is %f Hz\n", (double)gain_params->if_level);
	printf(" The Linearity mode is %d \n", (int)gain_params->linearity_mode);
}

void display_cal_data(cal_data_t* cal_data)
{
	int i,j;

	printf("IF CAL FREQ: \n");
	for (i = 0; i < IFCALFREQLEN; i++)
		printf("%f ", cal_data->if_cal_freq[i]);
	printf("\n IF CAL FREQ END \n");

	printf("IF REL GAIN: \n");
	for (i = 0; i < IFCALFREQLEN; i++)
		printf("%f ", cal_data->if_rel_gain_cal[i]);
	printf("\n IF REL GAIN END \n");

	printf("IF ATTEN CAL:\n");
	for (i = 0; i < CALATTENSTEPS; i++)
		printf("%f ", cal_data->if_atten_cal[i]);
	printf("\nIF ATTEN CAL END \n");

	printf("BYPASS FREQ: \n");
	for (i = 0; i < BYPASSFREQLEN; i++)
		printf("%f ", cal_data->bp_rf_cal_freq[i]);
	printf("\nBYPASS FREQ END \n");

	printf("BYPASS GAIN:\n");
	for (i = 0; i < BYPASSFREQLEN; i++)
		printf("%f ", cal_data->bp_abs_gain_cal[i]);
	printf("\nIF BYPASS GAIN \n");

	printf("RF CAL FREQ:\n");
	for (i = 0; i < RFCALFREQLEN; i++)
		printf("%f ", cal_data->rf_cal_freq[i]);
	printf("\n RF CAL FREQ END \n");

	printf("RF LSB ABS GAIN: \n");
	for (i = 0; i < RFCALFREQLEN; i++)
		printf("%f ", cal_data->rf_lsb_abs_gain_cal[i]);
	printf("\n RF LSB ABS GAIN END \n");

	printf("RF USB ABS GAIN: \n");
	for (i = 0; i < RFCALFREQLEN; i++)
		printf("%f ", cal_data->rf_usb_abs_gain_cal[i]);
	printf("\n RF USB ABS GAIN END \n");

	printf("RF AMP GAIN: \n");
	for (i = 0; i < RFCALFREQLEN; i++)
		printf("%f ", cal_data->rf_amp_gain_cal[i]);
	printf("\n RF AMP GAIN END \n");

	printf("IF ATTEN CAL: \n");
	for (i = 0; i < CALATTENSTEPS; i++)
	{
		for (j = 0; j < RFCALFREQLEN; j++)
		{
			printf("%f ", cal_data->rf_atten_cal[i][j]);
		}
		printf("\n");
	}
	printf("\n RF ATTEN CAL END \n");
}

int main()
{
	//define the device data parameters
	device_status_t dev_status;
	device_info_t dev_info;
	rf_params_t device_rf_params;
	sci_comm_interface_t comm_interface = USB_INT;
	int k_input;

	//init some device input/output values
	double rf_freq = 26e9; // rf freq = 29 GHz
	double if_freq = 1e9; // if at 1G
	uint64_t tmp_u64;
	float temperature;
	float conv_gain; //get the conversion gain


	// parameters to work with the USB device(s)
#ifndef MAXDEVICES
#define MAXDEVICES 50
#endif

#ifdef NIVISA
#define NAMESTRLEN SCI_RESOURCE_LENGTH
#else
#define NAMESTRLEN SCI_SN_LENGTH
#endif

	HANDLE dev_handle; //device handle
	int input; // user input to select the device found
	int num_of_devices; // the number of device types found
	char** device_list;  // 2D to hold serial numbers of the devices found 
	int i, status; // status reporting of functions


	gain_params_t conv_params;
	uint8_t auto_conv_enable;
	cal_data_t* cal_data = (cal_data_t*)malloc(sizeof(cal_data_t));

	/* 	Begin by looking for devices attached to the host
	*	=============================================================================================
	*/

	device_list = (char**)malloc(sizeof(char*) * MAXDEVICES); // MAXDEVICES serial numbers to search
	if(device_list)
		for (i = 0; i < MAXDEVICES; i++)
			device_list[i] = (char*)malloc(sizeof(char) * (NAMESTRLEN + 1)); // SCI SN has 8 char and "\0"

	printf("	Enter the interface (0) PXI, (1) USB, (2) RS232 \n");
	scanf(" %d", &k_input);
	getchar();
	comm_interface = k_input;
	status = mj2_SearchDevices(comm_interface, device_list, &num_of_devices); //searches for PSG device, with the specifie interface 

	if (num_of_devices == 0)
	{
#ifdef NIVISA
		printf("No signal core devices found or could be accessed\n");
#else
		printf("No signal core devices found or cannot obtain serial numbers\n");
#endif // NIVISA
		for (i = 0; i < MAXDEVICES; i++) free(device_list[i]);
		free(device_list);
		printf("\n\n Press ENTER to exit\n");
		getchar();
		return 1;
	}

	if(comm_interface == 0)
		printf("\n There are %d SignalCore %s PXI devices found. \n \n", num_of_devices, PXI_MODEL_NAME);
	else if (comm_interface == 1)
		printf("\n There are %d SignalCore %s USB devices found. \n \n", num_of_devices, USB_MODEL_NAME);
	else if (comm_interface == 2)
		printf("\n There are %d Ports  found. \n \n", num_of_devices);

	i = 0;
	while (i < num_of_devices)
	{
#ifdef NIVISA
		printf("	Device %d has resource value: %s \n", i + 1, device_list[i]);
#else
		if((comm_interface == 0) || (comm_interface == 1))
			printf("	Device %d has Serial Number: %s \n", i + 1, device_list[i]);
		else
			printf("	Device %d on Port: %s \n", i + 1, device_list[i]);
#endif
		i++;
	}
	/* 	*/
	printf("\n Enter the number of the device you wish to select : ");

	scanf(" %d", &input);
	getchar();
	if ((input < 1) || (input > num_of_devices))
	{
		printf(" No such device !!! exiting... \n");
		return 1;
	}

	/*	Open the selected device through the use of its serial number
	*/
	printf("\n Open the device ..........\n");
	status = mj2_OpenDevice(comm_interface, device_list[input - 1], 0, &dev_handle);
	if (dev_handle == NULL)
	{
#ifdef NIVISA
		printf("Device with serial number: %s cannot be opened.\n", device_list[input - 1]);
#else
		printf("Device with resource: %s cannot be opened.\n", device_list[input - 1]);
#endif
		printf("Please ensure your device is not locked by another process and connected\n");
		for (i = 0; i < MAXDEVICES; i++) free(device_list[i]);
		free(device_list);
		free(dev_handle); //dev_handle is allocated memory on OpenDevice(); 
		printf("\n\n Press ENTER to exit\n");
		getchar();
		return 1;
	}

	for (i = 0; i < MAXDEVICES; i++) free(device_list[i]);
	free(device_list); // Done with the deviceList

	printf("\n DEVICE OPENED!!..........\n");
	/*	Begin communication to the device
	*/
	printf("\n Reading the temperature..........\n");
	status = mj2_GetTemperature(dev_handle, &temperature); // Set to single tone mode, not sweep/list
	if (status != SCI_SUCCESS) return status;
	printf("\n The device temperature is %f\n", temperature);

	printf(" The RF frequency is set to %f Hz \n\n", rf_freq);
	status = mj2_SetFrequency(dev_handle, rf_freq); // Set Rf freq
	if (status != SCI_SUCCESS) return status;

	printf(" The IF frequency is set to %f Hz \n\n", if_freq);
	status = mj2_SetIfFrequency(dev_handle, if_freq); // Set Rf freq
	if (status != SCI_SUCCESS) return status;

	printf("\n Getting the device status..........\n");
	status = mj2_GetDeviceStatus(dev_handle, &dev_status); // Obtain the current status of the device
	if (status != SCI_SUCCESS) return status;
	display_device_status(&dev_status); // display the device status 

	printf("\nGetting device Info ..........\n");
	status = mj2_GetDeviceInfo(dev_handle, &dev_info);  // obtain calData
	if (status != SCI_SUCCESS) return status;
	display_device_info(&dev_info);

	printf("\nGetting RF parameters ..........\n");
	status = mj2_GetRfParameters(dev_handle, &device_rf_params);  // obtain calData
	if (status != SCI_SUCCESS) return status;
	display_rf_parameters(&device_rf_params);

	// set the device to automatic set the device on frequency changes
	auto_conv_enable = 1; //auto set device to achieve the conv_params.
	conv_params.linearity_mode = 0; //uses mixer level to do auto calculation
	conv_params.if_level = 4;
	conv_params.rf_level = -15;
	conv_params.mixer_level = -21;
	conv_params.auto_amp_ctrl = 1;

	status = mj2_SetAutoConversion(dev_handle, &conv_params, auto_conv_enable);
	if (status != SCI_SUCCESS)
		return status;

	//Get the conversion 
	status = mj2_CalcGain(dev_handle, &conv_gain);
	if (status != SCI_SUCCESS)
		return status;

	//Example using reg_read 
	status = mj2_RegRead(dev_handle, GET_DEVICE_PARAM, 0, &tmp_u64);
	printf("\n The RF Frequency using RegRead is %f \n\n", (double)tmp_u64 * 0.001); // change from mHz to Hz

	status = mj2_GetCalData(dev_handle, &cal_data);
	if (status != SCI_SUCCESS) return status;

	display_cal_data(cal_data);

	printf("\nClose Device ..........\n");
	status = mj2_CloseDevice(dev_handle); //function closes the device and frees the device Handle
	printf("\nClose Device status %d..........\n", status);
	printf("\n\n********** EXAMPLE DONE **********\n");

	printf("\n\n Press ENTER to exit\n");
	getchar();
    return 0;
}

