/*
 * Header functions mockingjay2 advance
 * SC5317A SC5318A
 *	
 *	Copyright (c) 2018 SignalCore Inc.
 *	
 *	Rev 2.0.0.0
*/

#ifndef MOCKINGJAY2_INTERNALS_H__
#define MOCKINGJAY2_INTERNALS_H__

#ifdef __cplusplus
extern "C"
{
#endif

/* Internal Function Prototypes */
/*! These functions are generally not required to be called explicitly by the user (application) */


/**	Store the conversion parameters used to calculate the gain parameters automatically to the device 
return: error code
input: 	gain_params struct
		auto_conv_params 
*/
SCISTATUS mj2adv_SaveAutoConvParams(HANDLE dev_handle, const gain_params_t* gain_params, uint8_t auto_conv_enable);

/**Function to automatically calculated the attenuator values based on the current gaim_params_t
return:	error code
input:	handle
output:	the attenuator struct with attenuator values
*/
SCISTATUS mj2adv_AutoCalcAttenValues(HANDLE dev_handle, attenuator_t* atten);

/* Read data of len bytes beginning at memStartAdd
return error code
input:	eeprom 0 or eeprom 1
		memory address to begin reading from
		len is the length of bytes to read
output:	byteData is the bytes read back
*/
SCISTATUS mj2adv_ReadEeprom(HANDLE dev_handle,
	uint8_t eeprom,
	uint32_t mem_start_add,
	uint32_t len,
	uint8_t *byte_data);

/*	Function reads and returns the entire Calibration EEPROM memory contents in raw bytes
	return:	error code
	output:	rawCalDataArray		a byte array with length equal to CALEEPROMSIZE
	Example: The calling function could dynamically allocate the array by doing so:
	unsigned char* rawCalDataArray = (unsigned char*)malloc(sizeof(unsigned char)*CALEEPROMSIZE);
*/
SCISTATUS mj2adv_FetchRawCalData(HANDLE dev_handle, uint8_t *raw_cal_data);

/*	Function to convert raw cal data to formatted data
*/
SCISTATUS mj2adv_RawToCalData(const uint8_t* raw_cal_data, cal_data_t* cal_data);

/** Allocate memory for structured calibration data
*/
SCISTATUS mj2adv_AllocateCalDataMemory(cal_data_t *cal_data);

/** deallocate memory for structured calibration data
*/
SCISTATUS mj2adv_DeallocateCalDataMemory(cal_data_t* cal_data);

/*Function to read calibration data into memory. If calibration data is no longer \
	required, call function mj2adv_ReleaseCalData() to free up memory. Closing the device will \
	also clear the memory. Functions that require calibration data will automatically call this function \ 
	to fetch it into memory if it isn't already in memory. The loading of cal data will cause a (one time) \
	delay (1-2 sec) of the function. 
return: error code
*/

SCISTATUS mj2adv_FetchCalData(HANDLE dev_handle);

/*Function to release data memory
*/
SCISTATUS mj2adv_ReleaseCalData(HANDLE dev_handle);

/*! Fetch the RF params from device into memory*/
SCISTATUS mj2adv_FetchRfParameters(HANDLE dev_handle);

/*! Fetch the gain parameters used for automatic calculation of the values used to set the device \
	accordingly. These are read into memory
*/
SCISTATUS mj2adv_FetchGainCalcParams(HANDLE dev_handle);

/*! Fetch other start up params into memory*/
SCISTATUS mj2adv_FetchStartUpParams(HANDLE dev_handle);

/*! Fetch the device information into memory
*/
SCISTATUS mj2adv_FetchDeviceInfo(HANDLE dev_handle);

#ifdef __cplusplus
}
#	endif

#	endif  /* MOCKINGJAY2_INTERNALS_H__ */	