/*	mj2_defs.h : include file for greenjay definitions
 *	
 * Author: SignalCore Inc
 * copyright SignalCore Inc
 * rev 2.2.0
 * 03/15/2020
*/

#pragma once

#ifndef MOCKINGJAY2_DEFS_H_
#define MOCKINGJAY2_DEFS_H_

#ifdef __cplusplus
extern "C"
{
#endif


#define SCI_PCI_VID						0x1BEB
#define SCI_PCI_PID						0x0034
#define SCI_USB_VID						0x277C
#define SCI_USB_PID						0x0028
#define SCI_PCI_VID_S					"0x1BEB"
#define SCI_PCI_PID_S					"0x0034"
#define SCI_USB_VID_S					"0x277C"
#define SCI_USB_PID_S					"0x0028"

#define MAX_PORT_NAME_LEN				0x012
#define SCI_SN_LENGTH					0x08
#define SCI_RESOURCE_LENGTH				0x20

#define PXI_MODEL_NAME					"SC5317A"
#define USB_MODEL_NAME					"SC5318A"			

#define SYS_ACTIVE_BYTE_POS				1
#define SYS_ACTIVE_BYTE_LOW				0
#define SYS_ACTIVE_BYTE_HIGH			1

#define PXI_CARD						0
#define USBSPI_CARD						1
#define USBRS232_CARD					2

	// Attenuator assignments
#define RF_ATTEN						0		
#define IF_ATTEN						1

/** 
 * Device types this API supports
 */

enum LOOPGAIN	/* pll loop gain*/
{
	NORMAL, /* factory default*/
	LOW 
};

#define EEPROM0							0	// EEPROM for factory and synthesizer cal data
#define EEPROM1							1	// EEPROM for conversion cal data

// Calibration EEPROM parameters dimensions
#define USEREEPROM						1
#define CALEEPROM						0
#define CALEEPROMSIZE					65536  // bytes 
#define CALDATASTARTADD					0x298  // start position for cal data on eeprom
#define CALDATALEN						0x5580 //total byte legnth of cal data
#define RFCALFREQLEN					83  // frequency point 6000MHz to 26500MHz step 250 MHz
#define CALATTENSTEPS					30  // attenuation steps 1-30 dB
#define IFCALFREQLEN					35  //freq points over the bandwidth  100MHz to 3500MHz, step 100 MHz
#define TEMPCOLEN						6	// 3 set of 2 2nd order coefficients, freq<12GHz, 12Ghz<freq<20GHz, freq>20 GHz
#define BYPASSFREQLEN					60	// 100 MHz to 6000 MHz, step 100 MHz
#define CALTEMP_ADD						0x298
#define TEMPCOEFF_ADD					0x29C
#define IFCALFREQ_ADD					0x398
#define IFRELGAIN_ADD					0x4B0
#define IFATTENCAL_ADD					0x5C8
#define BPCALFREQ_ADD					0x6B8
#define BPABSGAIN_ADD					0x7A8
#define RFCALFREQ_ADD					0x898
#define RFNONINVGAIN_ADD				0xBD0
#define RFINVGAIN_ADD					0xF08
#define RFAMPGAIN_ADD					0x1240
#define RFATTENCAL_ADD					0x1578

// DEVICE INFO
#define DEVINFOSNINT					0x00	//serial number address and interface
#define DEVINFOREV						0x01	// hardware and firmware revision address
#define DEVINFODATE						0x02	// manufactured and calibration date address


/*
 * Structure types to hold data
*/

typedef struct
{
	uint16_t year;
	uint8_t month;
	uint8_t day;
} scidate_t;

typedef struct device_info_s
{
	uint32_t product_sn;
	float fw_rev;
	float hw_rev;
	uint8_t dev_interface; //0=unassigned, 1= PXI, 2=USB&SPI, 3=USB&RS232
	scidate_t cal_date;
	scidate_t man_date;
} device_info_t;

typedef struct
{
	uint8_t lock_mode;
	uint8_t loop_gain;
	uint8_t auto_spur_suppress;
} synth_mode_t;

typedef struct
{
	uint8_t sum_pll_ld;			//lock status of main pll loop
	uint8_t crs_pll_ld;			//lock status of coarse offset pll loop (used only for harmonic mode)
	uint8_t fine_pll_ld;		//lock status of the dds tuned fine pll loop
	uint8_t vcxo_pll_ld;		//lock status of the 100 MHz VCXO pll loop
	uint8_t tcxo_pll_ld;		//lock status of the master 10 MHz TCXO pll loop
	uint8_t loop_gain;			//loop gain
} pll_status_t;

typedef struct
{
	uint8_t device_accessed;			//  indicates that the device is opened and accessed
	uint8_t ext_ref_detected;			// 1 indicates is a suitable external reference is detected
	uint8_t lock_ext_ref_enable;		// 2 indicates lock state of device to external source
	uint8_t lo_supply_enable;			// 3 indicates LO power status
	uint8_t ext_lo_enable;			// 4 LO drive select 1=internal, 0=external
	uint8_t ext_lo_path;			// 5 External LO input select 0 = rear LO input, 1 = front LO input
	uint8_t lo_mode;				// 6 mode of computing the LO frequency. 0=RF,IF,Inversion 1=LO direct
	uint8_t lo_doubler_enable;		// 7 Lo frequency x2 multiplier 
	uint8_t device_standby;			// 8 signal path power state
	uint8_t bypass_conv;				// 9 RFin-to-RFout port direct, no conversion,
	uint8_t if_out_enable;			// 10 IF output on/off
	uint8_t invert_spect;			// 11 Converted spectrum polarity    
	uint8_t rf_amp_enable;			// 12 RF amplifier on/off. on for better sensitivity
	uint8_t auto_gain_enable;			// 13 Device automatically adjusts the atten and preamp to set
									//gain of device close to the desired. This feature is disabled
									//by default. Use of the mathematically algorithms to set gain more
									//precisely
	uint8_t auto_amp_enable;			// 14 Device automatically controls the status of the RF preamplifier
	uint8_t pxi10_clk_enable;			// 15 Status of PXI clock output
} operate_status_t;

typedef struct
{
	pll_status_t pll_status;			//pll status
	operate_status_t operate_status;	//operating parameters
} device_status_t;

typedef struct signal_path_params
{
	uint8_t bypass_conversion;			//bypass_conversion - bypass the conversion stage, the RFinput  is directly passed onto the RF output
	uint8_t rf_amp_enable;					//rf_amp_enable - enables (1) or disables (0) rf amplifier
	uint8_t if_out_enable;					//if_out_enable- enables (1) or disables (0) if output port
	uint8_t invert_spectrum;			//invert the IF spectrum wrt to the input RF at the conversion stage. 
} signal_path_params_t;

typedef struct
{
	float rf_atten_value;				//RF Atten Value
	float if_atten_value;				//IF Atten Value
}	attenuator_t;

// Autogain parameters use to calculate the desired gain
typedef struct gainParams
{	
	float rf_level;					//expected rf level input
	float mixer_level;				//max desired level at mixer input
	float if_level;					//expected if level output
	uint8_t linearity_mode;			//linear modes: 0=mixer level control, 1=balance of snr and linearity, 2=better snr, best snr, better linearity, best linearity
	uint8_t auto_amp_ctrl;			//allows the preamplifier state to be changed to achieve the desired results
} gain_params_t;

typedef struct
{
	double rf_frequency;		// rf port frequency 
	double if_frequency;		// if port frequency
	double lo_frequency;		// lo frequency
	attenuator_t atten;
	signal_path_params_t path_params;
} rf_params_t;

typedef struct
{
	float cal_temp;
	float* temp_coeff;			//1D array of 2 coefficients c1, c2, 3 places  
	float* if_cal_freq;			// 1D array of IF cal frequencies (MHz) 100MHz to 5 GHz 100 MHz step
	float* if_rel_gain_cal;		// relative gain to 2 GHz center with non inverting spectrum
	float* if_atten_cal;			// 1D array of relative IF Atten cal, 1-30 dB, 1 dB step 
	float* bp_rf_cal_freq;			//1D array of bypass conversion frequencies 250MHz to 40 GHz, 250 MHz step
	float* bp_abs_gain_cal;			//1D array Bypass abs gain at each freq 
	float* rf_cal_freq;			// 1D array of rf cal frequencies 20 GHz to 40 GHz step 250 MHz
	float* rf_usb_abs_gain_cal;		// 1D array of rf abs gain at every rf freq
	float* rf_lsb_abs_gain_cal;		// 1D array of rf abs gain at every rf freq
	float* rf_amp_gain_cal;		//1D array of Relative RF Amp cal at rf freq
	float** rf_atten_cal;			//2D array (atten x freq) of RF atten 1 cal 
}	cal_data_t;

#ifdef __cplusplus
}
#endif

#	endif  /* MOCKINGJAY2_DEFS_H_ */	

